#include "G4RunManager.hh"

#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"

#include "DetectorConstruction.hh"
#include "PhysicsList.hh"
#include "PrimaryGeneratorAction.hh"
#include "SteppingAction.hh"

#include "SteppingVerbose.hh"

#ifdef G4UI_USE
#include "G4UIExecutive.hh"
#endif

#ifdef G4VIS_USE
 #include "G4VisExecutive.hh"
#endif

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
int main(int argc,char** argv) {
 
  // Michel's Verbose output class
  G4VSteppingVerbose::SetInstance(new SteppingVerbose);
    
  //Construct the default run manager
  G4RunManager* runManager = new G4RunManager;

  //set mandatory initialization classes
  //
  DetectorConstruction* detector  = new DetectorConstruction();
  runManager->SetUserInitialization(detector);
    
  PhysicsList* physics            = new PhysicsList();
  runManager->SetUserInitialization(physics);
    
  PrimaryGeneratorAction* primary = new PrimaryGeneratorAction(detector);
  runManager->SetUserAction(primary);

  //set user action classes
  //
  SteppingAction* step_action = new SteppingAction(detector);
  runManager->SetUserAction(step_action);
  
  // Visualization manager
  #ifdef G4VIS_USE
    G4VisManager* visManager = new G4VisExecutive;
    visManager->Initialize();
  #endif

  //Initialize Geant4 kernel
  //
  runManager->Initialize();

  //get the pointer to the User Interface manager 
  G4UImanager* UI = G4UImanager::GetUIpointer();  

  if (argc!=1) {   	// Batch mode 
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    UI->ApplyCommand(command+fileName);
  }
  
  else {          	// Define UI terminal for interactive mode
         
    // interactive mode : define UI session

#ifdef G4UI_USE
    G4UIExecutive* ui = new G4UIExecutive(argc, argv);

#ifdef G4VIS_USE
    UI->ApplyCommand("/control/execute vis.mac"); 
#endif

    ui->SessionStart();

#endif
  }

 //job termination
 //

#ifdef G4VIS_USE
  delete visManager;
#endif

 delete runManager;
 return 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo...... 
