#include "PhysicsList.hh"

#include "G4HadronElasticPhysics.hh"
#include "HadronPhysicsFTFP_BERT_HP.hh"
#include "G4DecayPhysics.hh"
#include "G4IonPhysics.hh"
#include "G4EmStandardPhysics_option3.hh"

#include "G4SystemOfUnits.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PhysicsList::PhysicsList():G4VModularPhysicsList()
{
  G4int verb = 1;
  
  // Em Physics
  fEmPhysicsList = new G4EmStandardPhysics_option3(verb);
  defaultCutValue = 1.0*mm;    

  // Decay Physics
  fDecayPhysicsList = new G4DecayPhysics(verb);

  // Hadron Elastic scattering
  fElaPhysicsList = new G4HadronElasticPhysics(verb);

  // Hadron Inelastic Physics
  fHadPhysicsList = new HadronPhysicsFTFP_BERT_HP(verb);    
  
  // Ion Physics
  fIonPhysicsList = new G4IonPhysics(verb);  
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PhysicsList::~PhysicsList()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PhysicsList::ConstructParticle()
{
  fEmPhysicsList ->ConstructParticle();
  fDecayPhysicsList ->ConstructParticle();
  fElaPhysicsList->ConstructParticle();
  fHadPhysicsList->ConstructParticle();
  fIonPhysicsList->ConstructParticle();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PhysicsList::ConstructProcess()
{
  AddTransportation();
  
  fEmPhysicsList ->ConstructProcess();   
  fDecayPhysicsList->ConstructProcess();   
  fElaPhysicsList->ConstructProcess();
  fHadPhysicsList->ConstructProcess();
  fIonPhysicsList->ConstructProcess();

  //ConstructDecay();
  AddStepMax();    
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PhysicsList::SetCuts()
{
  //G4VUserPhysicsList::SetCutsWithDefault method sets 
  //the default cut value for all particle types 
  //
  SetCutsWithDefault();
     
  if (verboseLevel>0) DumpCutValuesTable();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4PhysicsListHelper.hh"
#include "G4Decay.hh"

void PhysicsList::ConstructDecay()
{
  G4PhysicsListHelper* ph = G4PhysicsListHelper::GetPhysicsListHelper();
  
  // Create and add Decay Process
  //
  G4Decay* decay = new G4Decay();
  
  theParticleIterator->reset();
  
  while( (*theParticleIterator)() )
  {
    G4ParticleDefinition* particle = theParticleIterator->value();  
    if (decay->IsApplicable(*particle)) ph->RegisterProcess(decay, particle);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4ProcessManager.hh"
#include "G4StepLimiter.hh"

void PhysicsList::AddStepMax()
{
  // Step limitation seen as a process
  G4StepLimiter* stepMaxProcess = new G4StepLimiter();

  theParticleIterator->reset();
  while ((*theParticleIterator)()){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    if (particle->GetPDGCharge() != 0.0)
      pmanager ->AddDiscreteProcess(stepMaxProcess);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

